import * as sts from 'ts-simple-ast';
import SimpleProjectConstructor, { Node } from 'ts-simple-ast';
import * as ts from 'typescript';
import * as ts_module from 'typescript/lib/tsserverlibrary';
import { EvalContextUtil } from './evalCodeContextUtil';
export declare const EVAL_SELECTION_REFACTOR_ACTION_NAME = "plugin-ast-inspector-eval-selection";
export declare const EVAL_CURRENT_FUNCTION_BODY_REFACTOR_ACTION_NAME = "plugin-ast-inspector-eval-current-function-body";
/** context of evaluated code available in `c` variable */
export interface EvalContext {
    /** this is the whole typescript namespace as imported with `"import * as ts from 'typescript'` */
    ts: typeof ts;
    /** this is the whole ts-simple-ast namespace as imported with `import * as sts from 'ts-simple-ast'`. We are providing it as an utility because is much more high level than native typescript - you choose if you want ot work with it or not. */
    tsa: typeof sts;
    SimpleProjectConstructor: typeof SimpleProjectConstructor;
    /** The user selected or where his cursor is when he activated this refactor. It will be never undefined - at least it will be the SourceFile. The type is ts-simple-ast Node.  */
    node: Node;
    /** use it like console log to put debug strings that then will be printed back in the document */
    print(s: any): void;
    /** log messages back to tsserver (so host plugin and tsserver can see them). Is like a console.log() */
    log: (msg: string) => void;
    util: EvalContextUtil;
    /**
     * Entry point for the plugin execution context. from here you have access to the Project, the Program, current sourcefile, language service, plugin configuration and everything else regarding the context on where the host plugin is being executed. Take into account that everything obtained via `info` will be native typescript objects not ts-simple-ast. For example:
     *
     * ```
     * const sourceFile = c.config.info.project.getSourceFile(c.fileName)
     * const definition = c.info.languageService.getDefinitionAtPosition(toPosition(c.positionOrRange))
     * ```
     */
    info: ts_module.server.PluginCreateInfo;
    /** current file name as provided in plugin's `getEditsForRefactor`*/
    fileName: string;
    formatOptions: ts.FormatCodeSettings;
    /** cursor position or range from where the refactor suggestion was activated as provided in plugin's `getEditsForRefactor`*/
    positionOrRange: number | ts.TextRange;
    /** name of the activated refactor as provided in plugin's `getEditsForRefactor` */
    refactorName: string;
    /** name of the activated refactor's action as provided in plugin's `getEditsForRefactor` */
    actionName: string;
    project: sts.Project;
}
export interface EvalContextConfig {
    log: (str: string) => void;
    node: Node;
    info: ts_module.server.PluginCreateInfo;
    fileName: string;
    positionOrRange: number | ts.TextRange;
    formatOptions: ts.FormatCodeSettings;
    refactorName: string;
    actionName: string;
    project: sts.Project;
}
export declare function executeEvalCode(config: EvalContextConfig): void;
