"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const hrtime_now_1 = require("hrtime-now");
const sts = __importStar(require("ts-simple-ast"));
const ts_simple_ast_1 = __importDefault(require("ts-simple-ast"));
const ts = __importStar(require("typescript"));
const evalCodeContextUtil_1 = require("./evalCodeContextUtil");
// import { matchGlobalRegexWithGroupIndex } from './regex-groups-index';
// import { basename } from 'path';
// export const EVAL_CODE_IN_COMMENTS_REFACTOR_ACTION_NAME = `plugin-ast-inspector-eval-code-in-comments`
exports.EVAL_SELECTION_REFACTOR_ACTION_NAME = `plugin-ast-inspector-eval-selection`;
exports.EVAL_CURRENT_FUNCTION_BODY_REFACTOR_ACTION_NAME = `plugin-ast-inspector-eval-current-function-body`;
let _printed = [];
class EvalContextImpl {
    constructor(config) {
        this.ts = ts;
        this.tsa = sts;
        this.SimpleProjectConstructor = ts_simple_ast_1.default;
        this.util = new evalCodeContextUtil_1.EvalContextUtilImpl();
        this.info = config.info;
        this.fileName = config.fileName;
        this.formatOptions = config.formatOptions;
        this.positionOrRange = config.positionOrRange;
        this.refactorName = config.refactorName;
        this.actionName = config.actionName;
        this.log = config.log;
        this.node = config.node;
        _printed = [];
        this.project = config.project;
    }
    print(s) {
        _printed.push(s + '');
    }
}
function doEval(code, __context__) {
    // heads up - we are type/catch inside eval. If we let eval code throw exceptions this will impact not only this extension but all other plugins in the tsserver instance 
    const __result__ = {};
    const codeToEval = `
  try {
(function(c){
  ${code}
})(__context__);
}catch(ex){
  __result__.error = ex;
}
  `;
    try { // TODO: get how much time it took and print it back in the output
        eval(codeToEval); // TODO: eval code safely using node vm so security people dont complain
    }
    catch (ex) {
        __context__.log('executeEvalCode, eval error (outer): ' + ex + ' - ' + JSON.stringify(ex));
        __result__.errorOuter = ex;
    }
    __result__.output = _printed;
    return __result__;
}
const stackTrace = require('stack-trace');
function prettyPrintEvalResult(evalResult) {
    let output = '';
    if (evalResult.output) {
        output += `Output:\n${evalResult.output.join('\n')}\n`;
    }
    let error = '';
    if (evalResult.error) {
        var trace = stackTrace.parse(evalResult.error);
        error += `Error: (in)\n${evalResult.error}\nTrace: \n${stackTrace.parse(evalResult.error).map(i => `file:/${i.fileName || 'unknown.ts'}#${i.lineNumber},${i.columnNumber}) function ${i.functionName}`).join('\n')}\nOriginal Stack:\n ${evalResult.error.stack}\n`;
    }
    if (evalResult.errorOuter) {
        error += `Error: (out) \n${evalResult.errorOuter}\nStack:\n ${evalResult.errorOuter.stack}\n`;
    }
    return `\nvar __output = \`\n${error + output}\n\``; //TODO: escape ` ? 
}
function executeEvalCode(config) {
    const t0 = hrtime_now_1.now();
    const sourceFile = config.node.getSourceFile();
    const originalSourceFile = config.info.project.getSourceFile(sourceFile.getFilePath());
    // handle eval function body
    if (config.actionName === exports.EVAL_CURRENT_FUNCTION_BODY_REFACTOR_ACTION_NAME) {
        let targetFunction = (config.node.getKind() === ts.SyntaxKind.FunctionDeclaration ? config.node : undefined) || config.node.getFirstAncestorByKind(ts.SyntaxKind.FunctionDeclaration);
        if (!targetFunction) {
            // if we are not inside a function declaration we evaluate the body of the first function declaration in this file
            targetFunction = sourceFile.getFirstDescendantByKind(ts.SyntaxKind.FunctionDeclaration);
        }
        if (!targetFunction || !sts.TypeGuards.isFunctionDeclaration(targetFunction) || !targetFunction.getName()) {
            return;
        }
        // transpile the source file to js (so user is not restricted to js) and get the body text of the transpiled function */
        let codeToEval = targetFunction.getBody().getText();
        let configToEval = config;
        let transpiledSourceFile;
        try {
            const jsCode = sourceFile.getEmitOutput().getOutputFiles().find(f => f.getFilePath().endsWith('.js')).getText();
            const fileName = 'evaluated_' + new Date().getTime() + '.ts';
            transpiledSourceFile = config.project.createSourceFile(fileName, jsCode);
            const body = transpiledSourceFile.getFunction(targetFunction.getName()).getBody().getText();
            if (body) {
                codeToEval = transpiledSourceFile.getText() + ';\n' + targetFunction.getName() + '();';
                configToEval = Object.assign({}, config, { sourceFile: transpiledSourceFile });
            }
        }
        catch (ex) {
            config.log(' log failed to transpile - we continue evaluating original function though. Error: ' + ex + ' - ' + ex.stack);
            //TODO:  log failed to transpile - we continue evaluating original function though
        }
        const text = evalCodeAndPrintResult(configToEval, codeToEval);
        sourceFile.insertText(targetFunction.getEnd(), text);
        if (transpiledSourceFile) {
            transpiledSourceFile.deleteImmediatelySync();
        }
    }
    // handle eval selected code
    else if (config.actionName === exports.EVAL_SELECTION_REFACTOR_ACTION_NAME &&
        typeof config.positionOrRange.pos === 'number') {
        const range = config.positionOrRange;
        const text = evalCodeAndPrintResult(config, originalSourceFile.getFullText().substring(range.pos, range.end));
        sourceFile.insertText(range.end, text);
        return;
    }
    // // handle eval code in comments
    // else {
    //   const regex = /\/\*\*\*@\s*([^@]+)\s*(@\*\*\*\/)/gim
    //   const result = matchGlobalRegexWithGroupIndex(regex, originalSourceFile.getFullText())
    //   config.log('executeEvalCode apply matchGlobalRegexWithGroupIndex result ' + JSON.stringify(result, null, 2))
    //   const toPrint = result && result.length && result.map(match => {
    //     return { text: evalCodeAndPrintResult(config, match[0].value), printPosition: match[1].end }
    //   })
    //   config.log('executeEvalCode apply return true and toPrint == ' + toPrint ? JSON.stringify(toPrint, null, 2) : 'undefined')
    //   if (!toPrint) {
    //     sourceFile.insertText(config.node.getSourceFile().getEnd(), evalHelpText)
    //   }
    //   else {
    //     toPrint.forEach(content => {
    //       sourceFile.insertText(content.printPosition, content.text)
    //     })
    //     config.log('executeEvalCode took ' + timeFrom(t0))
    //   }
    // }
}
exports.executeEvalCode = executeEvalCode;
function evalCodeAndPrintResult(config, code) {
    const context = new EvalContextImpl(config);
    config.log('executeEvalCode evaluating selected code: ' + code);
    const result = doEval(code, context); //TODO: not logging time & error
    const text = prettyPrintEvalResult(result);
    config.log('executeEvalCode after evaluating selected code result is: ' + JSON.stringify(result, null, 2));
    return text;
}
// const evalHelpText = `
// /***@ 
//  // For evaluating code you can use a comment with a format like this one, (see how starts with "/*** followed
//  // by "at") You could have many of these comments as this one as long they contain VALID JAVASCRIPT (this is
//  // why we use line comments inside for this internal comments) IMPORTANT: make sure you save the file before
//  // evaluating code - if not the content in editor buffer will be different (older) than filesystem (most
//  // editors handle this OK but be advised) 
//  //You have a "c" variable with a context object with useful utilities, among others:TODO: IEvalContext apidocs)
// // ts: typeof ts                             whole typescript namespace available
// // sts: typeof tsa                           whole ts-simple-ast namespace available
// // node: Node                                node selected by user when activated this refactor
// // print(s): void                            print text back here a analog to console.log 
// // printAst (node:Node|ts.Node): string      pretty prints AST structure of given node to understand it 
// // This is the code will be executed here : 
// c.print(\`
// Hello from editor. Using typescript version: \${c.ts.version}
// Selected node by user is a \${c.node.getKindName()} and its parent's text is "\${c.node.getParent().getText()}"
// The AST structure of this file:  
// \${c.printAst(c.node.getSourceFile())}
// \`)
// @***/
// `
