"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const hrtime_now_1 = require("hrtime-now");
const typescript_ast_util_1 = require("typescript-ast-util");
const ts_simple_ast_extra_1 = require("ts-simple-ast-extra");
const evalCode_1 = require("./evalCode");
const ts = __importStar(require("typescript"));
const PLUGIN_NAME = 'typescript-plugin-ast-inspector';
const PRINT_AST_REFACTOR_ACTION_NAME = `${PLUGIN_NAME}-print-ast-refactor-action`;
const PRINT_PARENT_NODES_REFACTOR_ACTION_NAME = `${PLUGIN_NAME}-print-inheritance-refactor-action`;
// const PRINT_SELECTED_NODE_REFACTOR_ACTION_NAME = `${PLUGIN_NAME}-print-selected-node`
// function installMagic(info: ts_module.server.PluginCreateInfo){
//   const transformer = <T extends ts.SourceFile>(context: ts.TransformationContext) => {
//     return  rootNode => {
//       function visit(node: ts.Node): ts.Node {
//         node = ts.visitEachChild(node, visit, context);
//         if (ts.isPropertyAccessExpression(node) && node.name &&
//           node.name.getText() === 'foo') {
//           return node.expression
//         }
//         return node;
//       }
//       return ts.visitNode(rootNode, visit);
//     }
//   }
//  info.languageServiceHost.getCustomTransformers().before = 
//   (info.languageServiceHost.getCustomTransformers().before || []).concat([transformer]);
//   log('MAGIC DONE: '+info.languageServiceHost.getCustomTransformers().before.length)
// }
let info;
let log;
let nodeAtCursor;
function getApplicableRefactors(fileName, positionOrRange, userPreferences) {
    const refactors = info.languageService.getApplicableRefactors(fileName, positionOrRange, userPreferences) || [];
    const sourceFile = info.languageService.getProgram().getSourceFile(fileName);
    if (!sourceFile) {
        return refactors;
    }
    nodeAtCursor = typescript_ast_util_1.findChildContainingRangeLight(sourceFile, typescript_ast_util_1.positionOrRangeToRange(positionOrRange));
    if (!nodeAtCursor) {
        nodeAtCursor = typescript_ast_util_1.findChildContainingPosition(sourceFile, typescript_ast_util_1.positionOrRangeToNumber(positionOrRange));
    }
    if (!nodeAtCursor) {
        return refactors;
    }
    const fn = typescript_ast_util_1.findAscendant(nodeAtCursor, ts.isFunctionDeclaration, true);
    let actions = [];
    if (fn) {
        actions.push({ name: evalCode_1.EVAL_CURRENT_FUNCTION_BODY_REFACTOR_ACTION_NAME, description: `Evaluate ${fn.name && fn.name.getText() || 'current'} function` });
    }
    if (positionOrRange.pos) {
        actions.push({ name: evalCode_1.EVAL_SELECTION_REFACTOR_ACTION_NAME, description: 'Evaluate selected code' });
    }
    actions = actions.concat([
        { name: PRINT_AST_REFACTOR_ACTION_NAME, description: 'Print AST of selected ' + typescript_ast_util_1.getKindName(nodeAtCursor.kind) },
        { name: PRINT_PARENT_NODES_REFACTOR_ACTION_NAME, description: 'Print parent nodes of selected ' + typescript_ast_util_1.getKindName(nodeAtCursor.kind) }
    ]);
    refactors.push({
        name: `${PLUGIN_NAME}-ast-inspector`,
        description: 'ast-inspector',
        actions
    });
    return refactors;
}
function getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName) {
    if (actionName === PRINT_AST_REFACTOR_ACTION_NAME && nodeAtCursor) {
        return printAst(nodeAtCursor, fileName);
    }
    else if (actionName === PRINT_PARENT_NODES_REFACTOR_ACTION_NAME && nodeAtCursor) {
        return printParentNodes(nodeAtCursor, fileName);
    }
    else if ( /* actionName === EVAL_CODE_IN_COMMENTS_REFACTOR_ACTION_NAME ||  */actionName === evalCode_1.EVAL_SELECTION_REFACTOR_ACTION_NAME || actionName === evalCode_1.EVAL_CURRENT_FUNCTION_BODY_REFACTOR_ACTION_NAME) {
        return evalCode(fileName, positionOrRange, formatOptions, refactorName, actionName);
    }
}
function evalCode(fileName, positionOrRange, formatOptions, refactorName, actionName) {
    const t0 = hrtime_now_1.now();
    // const createSimpleASTProjectT0 = now()
    const simpleProject = ts_simple_ast_extra_1.createSimpleASTProject(info.project);
    const sourceFile = simpleProject.getSourceFile(fileName);
    const simpleNode = sourceFile.getDescendantAtPos(typescript_ast_util_1.positionOrRangeToNumber(positionOrRange)) || sourceFile;
    // info.project.projectService.logger.info(`${PLUGIN_NAME} evalCode createSimpleASTProject took ${timeFrom(createSimpleASTProjectT0)}`)
    const fixapplyT0 = hrtime_now_1.now();
    try {
        evalCode_1.executeEvalCode({ log, node: simpleNode, fileName, info, positionOrRange, formatOptions, refactorName, actionName, project: simpleProject });
    }
    catch (error) {
        info.project.projectService.logger.info(`${PLUGIN_NAME} evalCode executeEvalCode error ${error} \n ${error.stack}`);
    }
    info.project.projectService.logger.info(`${PLUGIN_NAME} evalCode executeEvalCode took ${hrtime_now_1.timeFrom(fixapplyT0)}`);
    const saveSyncT0 = hrtime_now_1.now();
    // sourceFile.emit() 
    simpleProject.saveSync();
    info.project.projectService.logger.info(`${PLUGIN_NAME} evalCode saveSync took ${hrtime_now_1.timeFrom(saveSyncT0)}`);
    info.project.projectService.logger.info(`${PLUGIN_NAME} evalCode total time took ${hrtime_now_1.timeFrom(t0)}`);
    return;
}
function printAst(nodeAtCursor, fileName) {
    let newText = `\`
AST of selected ${typescript_ast_util_1.getKindName(nodeAtCursor.kind)}:  
${typescript_ast_util_1.dumpAst(nodeAtCursor)}
\``;
    return {
        edits: [{
                fileName,
                textChanges: [{
                        span: { start: nodeAtCursor.getSourceFile().getEnd(), length: newText.length },
                        newText: newText
                    }],
            }],
        renameFilename: undefined,
        renameLocation: undefined,
    };
}
function printParentNodes(nodeAtCursor, fileName) {
    let node = nodeAtCursor;
    const arr = [];
    let newText = '';
    newText += `\nParent AST Nodes of selected ${typescript_ast_util_1.getKindName(nodeAtCursor.kind)}:\n`;
    do {
        if (!node) {
            break;
        }
        arr.push(node);
        node = node.parent;
    } while (node);
    let level = 1;
    newText += arr.reverse().map(n => {
        return typescript_ast_util_1.printNode(n, -1, level++) + ', starts: ' + n.getStart();
    }).join('\n');
    newText = '\n`\n' + newText + '\n`\n';
    return {
        edits: [{
                fileName,
                textChanges: [{
                        span: { start: nodeAtCursor.getSourceFile().getEnd(), length: newText.length },
                        newText: newText
                    }],
            }],
        renameFilename: undefined,
        renameLocation: undefined,
    };
}
module.exports = function init(modules) {
    function create(anInfo) {
        info = anInfo;
        log = function (msg) {
            info.project.projectService.logger.info(`${PLUGIN_NAME} ${msg}`);
        };
        info.project.projectService.logger.info(`${PLUGIN_NAME} created`);
        const proxy = Object.create(null);
        for (let k of Object.keys(info.languageService)) {
            const x = info.languageService[k];
            proxy[k] = (...args) => x.apply(info.languageService, args);
        }
        proxy.getApplicableRefactors = getApplicableRefactors;
        proxy.getEditsForRefactor = getEditsForRefactor;
        // installMagic(info)
        return proxy;
    }
    return { create };
};
